// Redeye.cpp: implementation of the CRedeye class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ILBlinky.h"
#include "Redeye.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#define EOT		4							// End Of Text
#define LF		10							// Line Feed
#define ESC		27							// ESCape

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

//BYTE byTranslation[256], 
CRedeye::CRedeye(const std::vector<BYTE> vbyTranslation,LPCTSTR szUdpServer, UINT uUdpPort, bool bAutoExtAddr)
	: m_vbyTranslation(vbyTranslation)
	, m_strUdpServer(szUdpServer)
	, m_uUdpPort(uUdpPort)
	, CIldevBase(bAutoExtAddr)
{
	// HP-IL data and variables
	m_byAID = 47;							// Accessory ID = printer
	m_byDEFADDR = 31;						// default address after AAU
	m_strDID = _T("BLINKY");				// Device ID

	m_ip_addr.s_addr = INADDR_NONE;			// invalidate saved UDP address

	m_bEsc = false;							// not ESC sequence
	m_byGraphLength = 0;					// no remaining graphic bytes
}

CRedeye::~CRedeye()
{

}

void CRedeye::SetInfraredConfiguration(LPCTSTR szUdpServer, UINT uUdpPort)
{
	m_strUdpServer = szUdpServer;			// new UDP server
	m_uUdpPort = uUdpPort;

	m_ip_addr.s_addr = INADDR_NONE;			// invalidate saved UDP address
	return;
}

//
// convert text only
//
BYTE CRedeye::ConvertText(BYTE byData)
{
	do
	{
		// normal character for translation
		if (!m_bEsc && m_byGraphLength == 0)
		{
			// translate
			ASSERT(byData < m_vbyTranslation.size());
			byData = m_vbyTranslation[byData];
		}

		// check for begin of ESC sequence
		if (byData == ESC && !m_bEsc && m_byGraphLength == 0)
		{
			m_bEsc = true;					// ESC sequence mode
			break;
		}

		if (m_bEsc)							// byte is ESC command
		{
			switch (byData)
			{
			case 255: // reset
			case 254: // start self test
			case 253: // start printing expanded characters
			case 252: // return to normal-width characters
			case 251: // start printing underlined characters
			case 250: // stop underlining
			case 249: // ECMA 94 character set
			case 248: // Roman 8 character set
				break;
			default:
				// graphic data
				if (byData >= 1 && byData <= 166)
				{
					// remaining graphic bytes
					m_byGraphLength = byData;
				}
			}
			m_bEsc = false;
			break;
		}

		if (m_byGraphLength > 0)			// in graphic mode
		{
			--m_byGraphLength;				// graphic character printed
		}
	}
	while (false);
	return byData;
}

bool CRedeye::SendByteUdp(BYTE byData)
{
	WSADATA     wsd;
	SOCKET      sClient;
	SOCKADDR_IN sServer;

	bool bErr = true;

	VERIFY(WSAStartup(MAKEWORD(1,1),&wsd) == 0);

	if (m_ip_addr.s_addr == INADDR_NONE)	// IP address not specified
	{
		LPSTR lpszIpAddr;

		#if defined _UNICODE
			DWORD dwLength = lstrlen(m_strUdpServer) + 1;

			if ((lpszIpAddr = (LPSTR) _alloca(dwLength)) == NULL)
				return TRUE;				// server not found

			WideCharToMultiByte(CP_ACP, WC_COMPOSITECHECK,
								(LPCTSTR) m_strUdpServer, dwLength,
								lpszIpAddr, dwLength, NULL, NULL);
		#else
			lpszIpAddr = const_cast<LPSTR>((LPCTSTR) m_strUdpServer);
		#endif

		m_ip_addr.s_addr = inet_addr(lpszIpAddr);

		// not a valid ip address -> try to get ip address from name server
		if (m_ip_addr.s_addr == INADDR_NONE)
		{
			PHOSTENT host = gethostbyname(lpszIpAddr);
			if (host == NULL)
			{
				return true;				// server not found
			}

			m_ip_addr.s_addr = ((PIN_ADDR) host->h_addr_list[0])->s_addr;
		}
	}

	// create UDP socket
	if ((sClient = socket(AF_INET, SOCK_DGRAM, 0)) != INVALID_SOCKET)
	{
		sServer.sin_family = AF_INET;
		sServer.sin_port = htons(m_uUdpPort);
		sServer.sin_addr.s_addr = m_ip_addr.s_addr;

		// transmit data byte
		bErr = sendto(sClient, (LPCCH) &byData, sizeof(byData), 0, (LPSOCKADDR) &sServer, sizeof(sServer)) == SOCKET_ERROR;
		closesocket(sClient);
	}

	WSACleanup();							// cleanup network stack
	return bErr;
}

VOID CRedeye::ClearDevice()
{
	SendByteUdp(27);						// ESC
	SendByteUdp(255);						// printer reset
	m_bEsc = false;							// not ESC sequence
	m_byGraphLength = 0;					// no remaining graphic bytes
	return;
}

VOID CRedeye::InData(WORD wFrame)			// listener transfer
{
	BYTE byData = static_cast<BYTE>(wFrame);
	byData = ConvertText(byData);			// adjust text
	SendByteUdp(byData);
	return;
}





